@@ -1,6 +1,6 @@
 /*
  * Copyright 2013-2016 Canonical Ltd.
- * Copyright 2021 Robert Tari
+ * Copyright 2021-2023 Robert Tari
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 3, as published
@@ -200,14 +200,6 @@ device_compare_func (gconstpointer ga, gconstpointer gb)
     }
 
   state = UP_DEVICE_STATE_DISCHARGING;
-
-  /* discharging items with more than 10% remaining always lose */
-  if (!ret && (((a_state == state) && !a_time && (a_percentage > 10))))
-      ret = 1;
-
-  if (!ret && (((b_state == state) && !b_time && (b_percentage > 10))))
-      ret = -1;
-
   if (!ret && (((a_state == state) && a_time) ||
                ((b_state == state) && b_time)))
     {
@@ -361,21 +353,6 @@ calculate_device_state_action_state (IndicatorPowerService * self)
   return g_variant_new_string(device_state_to_string(device_state));
 }
 
-static GVariant*
-calculate_battery_level_action_state (IndicatorPowerService * self)
-{
-  const priv_t * const p = self->priv;
-  guint32 battery_level;
-
-  if (p->primary_device == NULL)
-    battery_level = 0;
-  else
-    battery_level = (guint32)(indicator_power_device_get_percentage (p->primary_device) + 0.5);
-
-  return g_variant_new_uint32 (battery_level);
-}
-
-
 /***
 ****
 ****  HEADER SECTION
@@ -489,7 +466,7 @@ create_header_state (IndicatorPowerService * self)
             g_free (title);
         }
 
-      if ((icon = indicator_power_device_get_gicon (p->primary_device)))
+      if ((icon = indicator_power_device_get_gicon (p->primary_device, TRUE, TRUE)))
         {
           GVariant * serialized_icon = g_icon_serialize (icon);
 
@@ -502,6 +479,16 @@ create_header_state (IndicatorPowerService * self)
           g_object_unref (icon);
         }
     }
+    else if (ayatana_common_utils_is_lomiri())
+    {
+        g_variant_builder_add (&b, "{sv}", "label", g_variant_new_string ("Fake battery"));
+        g_variant_builder_add (&b, "{sv}", "accessible-desc", g_variant_new_string ("Fake battery"));
+        GIcon *pIcon = g_themed_icon_new_with_default_fallbacks ("battery-missing-symbolic");
+        GVariant *pIconSerialised = g_icon_serialize (pIcon);
+        g_object_unref (pIcon);
+        g_variant_builder_add (&b, "{sv}", "icon", pIconSerialised);
+        g_variant_unref (pIconSerialised);
+    }
 
   return g_variant_builder_end (&b);
 }
@@ -513,82 +500,85 @@ create_header_state (IndicatorPowerService * self)
 ****
 ***/
 
-static void
-append_device_to_menu (GMenu * menu, const IndicatorPowerDevice * device, int profile)
-{
-  const UpDeviceKind kind = indicator_power_device_get_kind (device);
-
-  if (kind != UP_DEVICE_KIND_LINE_POWER)
-  {
-    char * label;
-    GMenuItem * item;
-    GIcon * icon;
-
-    label = indicator_power_device_get_readable_text (device);
-    item = g_menu_item_new (label, NULL);
-    g_free (label);
-
-    g_menu_item_set_attribute (item, "x-ayatana-type", "s", "org.ayatana.indicator.basic");
-
-    if ((icon = indicator_power_device_get_gicon (device)))
-      {
-        GVariant * serialized_icon = g_icon_serialize (icon);
-
-        if (serialized_icon != NULL)
-          {
-            g_menu_item_set_attribute_value (item,
-                                             G_MENU_ATTRIBUTE_ICON,
-                                             serialized_icon);
-            g_variant_unref (serialized_icon);
-          }
-
-        g_object_unref (icon);
-      }
-
-    if (profile == PROFILE_DESKTOP)
-      {
-        g_menu_item_set_action_and_target(item, "indicator.activate-statistics", "s",
-                                          indicator_power_device_get_object_path (device));
-      }
-
-    g_menu_append_item (menu, item);
-    g_object_unref (item);
-  }
-}
-
-
 static GMenuModel *
-create_desktop_devices_section (IndicatorPowerService * self, int profile)
+create_devices_section (IndicatorPowerService * self, int profile)
 {
-  GList * l;
-  GMenu * menu = g_menu_new ();
+    GMenu * menu = g_menu_new ();
+    GList * l;
 
-  for (l=self->priv->devices; l!=NULL; l=l->next)
-    append_device_to_menu (menu, l->data, profile);
+    for (l=self->priv->devices; l!=NULL; l=l->next)
+    {
+        IndicatorPowerDevice *device = l->data;
+        const UpDeviceKind kind = indicator_power_device_get_kind (device);
 
-  return G_MENU_MODEL (menu);
+        if (kind != UP_DEVICE_KIND_LINE_POWER)
+        {
+            gchar *sLabel = NULL;
+
+            if (kind == UP_DEVICE_KIND_BATTERY)
+            {
+                if (!ayatana_common_utils_is_lomiri())
+                {
+                    sLabel = indicator_power_device_get_readable_text (device, FALSE);
+                }
+                else
+                {
+                    sLabel = g_strdup (_("Charge level"));
+                }
+            }
+            else
+            {
+                sLabel = indicator_power_device_get_readable_text (device, TRUE);
+            }
+
+            gchar *sIndicator = NULL;
+            gchar *sProperty = NULL;
+
+            if (!ayatana_common_utils_is_lomiri())
+            {
+                sIndicator = "org.ayatana.indicator.level";
+                sProperty = "x-ayatana-level";
+            }
+            else
+            {
+                sIndicator = "org.ayatana.indicator.progress";
+                sProperty = "x-ayatana-progress";
+            }
+
+            GMenuItem * item = g_menu_item_new (sLabel, NULL);
+            g_free (sLabel);
+            g_menu_item_set_attribute (item, "x-ayatana-type", "s", sIndicator);
+            guint16 battery_level = (guint16)(indicator_power_device_get_percentage (device) + 0.5);
+            g_menu_item_set_attribute (item, sProperty, "q", battery_level);
+
+            if (!ayatana_common_utils_is_lomiri())
+            {
+                GIcon * icon = indicator_power_device_get_gicon (device, FALSE, FALSE);
+
+                if (icon)
+                {
+                    GVariant * serialized_icon = g_icon_serialize (icon);
+
+                    if (serialized_icon != NULL)
+                    {
+                        g_menu_item_set_attribute_value (item, G_MENU_ATTRIBUTE_ICON, serialized_icon);
+                        g_variant_unref (serialized_icon);
+                    }
+
+                    g_object_unref (icon);
+                }
+
+                g_menu_item_set_action_and_target(item, "indicator.activate-statistics", "s", indicator_power_device_get_object_path (device));
+            }
+
+            g_menu_append_item (menu, item);
+            g_object_unref (item);
+        }
+    }
+
+    return G_MENU_MODEL (menu);
 }
 
-/* https://wiki.ubuntu.com/Power#Phone
- * The spec also discusses including an item for any connected bluetooth
- * headset, but bluez doesn't appear to support Battery Level at this time */
-static GMenuModel *
-create_phone_devices_section (IndicatorPowerService * self G_GNUC_UNUSED)
-{
-  GMenu * menu;
-  GMenuItem *item;
-
-  menu = g_menu_new ();
-
-  item = g_menu_item_new (_("Charge level"), "indicator.battery-level");
-  g_menu_item_set_attribute (item, "x-ayatana-type", "s", "org.ayatana.indicator.progress");
-  g_menu_append_item (menu, item);
-  g_object_unref (item);
-
-  return G_MENU_MODEL (menu);
-}
-
-
 /***
 ****
 ****  SETTINGS SECTION
@@ -674,7 +664,7 @@ create_phone_settings_section(IndicatorPowerService * self)
 
   if (ab_supported)
     {
-      item = g_menu_item_new(_("Adjust brightness automatically"), "indicator.auto-brightness");
+      item = g_menu_item_new(_("Adjust brightness automatically"), "indicator.auto-brightness(true)");
       g_menu_item_set_attribute(item, "x-ayatana-type", "s", "org.ayatana.indicator.switch");
       g_menu_append_item(section, item);
       g_object_unref(item);
@@ -682,13 +672,13 @@ create_phone_settings_section(IndicatorPowerService * self)
 
   if (flashlight_supported())
   {
-    item = g_menu_item_new(_("Flashlight"), "indicator.flashlight");
+    item = g_menu_item_new(_("Flashlight"), "indicator.flashlight(true)");
     g_menu_item_set_attribute(item, "x-ayatana-type", "s", "org.ayatana.indicator.switch");
     g_menu_append_item(section, item);
     g_object_unref(item);
     if (flashlight_activated())
     {
-      item = g_menu_item_new(_("Warning: Heavy use can damage the LED!"), "indicator.flashlight");
+      item = g_menu_item_new(_("Warning: Heavy use can damage the LED!"), "indicator.flashlight(true)");
       g_menu_append_item(section, item);
       g_object_unref(item);
     }
@@ -736,8 +726,8 @@ rebuild_now (IndicatorPowerService * self, guint sections)
 
   if (sections & SECTION_DEVICES)
     {
-      rebuild_section (desktop->submenu, 0, create_desktop_devices_section (self, PROFILE_DESKTOP));
-      rebuild_section (greeter->submenu, 0, create_desktop_devices_section (self, PROFILE_DESKTOP_GREETER));
+      rebuild_section (desktop->submenu, 0, create_devices_section (self, PROFILE_DESKTOP));
+      rebuild_section (greeter->submenu, 0, create_devices_section (self, PROFILE_DESKTOP_GREETER));
     }
 
   if (sections & SECTION_SETTINGS)
@@ -771,17 +761,17 @@ create_menu (IndicatorPowerService * self, int profile)
   switch (profile)
     {
       case PROFILE_PHONE:
-        sections[n++] = create_phone_devices_section (self);
+        sections[n++] = create_devices_section (self, PROFILE_PHONE);
         sections[n++] = create_phone_settings_section (self);
         break;
 
       case PROFILE_DESKTOP:
-        sections[n++] = create_desktop_devices_section (self, PROFILE_DESKTOP);
+        sections[n++] = create_devices_section (self, PROFILE_DESKTOP);
         sections[n++] = create_desktop_settings_section (self);
         break;
 
       case PROFILE_DESKTOP_GREETER:
-        sections[n++] = create_desktop_devices_section (self, PROFILE_DESKTOP_GREETER);
+        sections[n++] = create_devices_section (self, PROFILE_DESKTOP_GREETER);
         break;
     }
 
@@ -910,12 +900,6 @@ init_gactions (IndicatorPowerService * self)
   g_action_map_add_action (G_ACTION_MAP(p->actions), G_ACTION(a));
   p->header_action = a;
 
-  /* add the power-level action */
-  a = g_simple_action_new_stateful ("battery-level", NULL, calculate_battery_level_action_state(self));
-  g_simple_action_set_enabled (a, FALSE);
-  g_action_map_add_action (G_ACTION_MAP(p->actions), G_ACTION(a));
-  p->battery_level_action = a;
-
   /* add the charge state action */
   a = g_simple_action_new_stateful ("device-state", NULL, calculate_device_state_action_state(self));
   g_simple_action_set_enabled (a, FALSE);
@@ -923,7 +907,9 @@ init_gactions (IndicatorPowerService * self)
   p->device_state_action = a;
 
   /* add the auto-brightness action */
-  a = g_simple_action_new_stateful("auto-brightness", NULL, g_variant_new_boolean(FALSE));
+  GVariantType *pType = g_variant_type_new ("b");
+  a = g_simple_action_new_stateful ("auto-brightness", pType, g_variant_new_boolean (FALSE));
+  g_variant_type_free (pType);
   g_object_bind_property_full(p->brightness, "auto-brightness",
                               a,  "state",
                               G_BINDING_SYNC_CREATE|G_BINDING_BIDIRECTIONAL,
@@ -933,9 +919,11 @@ init_gactions (IndicatorPowerService * self)
   g_action_map_add_action(G_ACTION_MAP(p->actions), G_ACTION(a));
 
   /* add the flashlight action */
-  a = g_simple_action_new_stateful("flashlight", NULL, g_variant_new_boolean(FALSE));
+  pType = g_variant_type_new ("b");
+  a = g_simple_action_new_stateful ("flashlight", pType, g_variant_new_boolean (FALSE));
+  g_variant_type_free (pType);
   g_action_map_add_action (G_ACTION_MAP(p->actions), G_ACTION(a));
-  g_signal_connect(a, "activate", G_CALLBACK(toggle_flashlight_action), self);
+  g_signal_connect(a, "change-state", G_CALLBACK(toggle_flashlight_action), self);
 
   /* add the brightness action */
   a = g_simple_action_new_stateful ("brightness", NULL, action_state_for_brightness (self));
@@ -1083,9 +1071,6 @@ on_devices_changed (IndicatorPowerService * self)
   else
     indicator_power_notifier_set_battery (p->notifier, NULL);
 
-  /* update the battery-level action's state */
-  g_simple_action_set_state (p->battery_level_action, calculate_battery_level_action_state(self));
-
   /* update the device-state action's state */
   g_simple_action_set_state (p->device_state_action, calculate_device_state_action_state(self));
 
@@ -1440,6 +1425,7 @@ create_totalled_battery_device (const GList * devices)
 
       device = indicator_power_device_new (NULL,
                                            UP_DEVICE_KIND_BATTERY,
+                                           NULL,
                                            percent,
                                            state,
                                            time_left,
